@@ -4,6 +4,6 @@
 
 namespace xs { namespace lib {
 
-SV* clone (pTHX_ SV* source, bool cross);
+SV* clone (pTHX_ SV* source, bool cross = false);
 
 }}
